/** BEGIN COPYRIGHT BLOCK
 * Copyright (c) 2006  Red Hat, Inc. All rights reserved.
 * 
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions of
 * the Apache License, 2.0.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY expressed or implied, including the implied warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  A copy of this
 * license is available at http://www.opensource.org/licenses.  Any Red Hat
 * trademarks that are incorporated in the source code or documentation are
 * not subject to the Apache License, 2.0 and may only be used or replicated
 * with the express permission of Red Hat, Inc.
 * 
 * Red Hat Author(s): Rob Crittenden
 * END COPYRIGHT BLOCK **/
/* 
 *
 * client.cpp - A wrapper around generic URL-getters
 * 
 */

#include <string.h>

#include "client.h"
#include "client_err.h"
#include "rev_core.h"

typedef struct client_error_t {
    int errorNumber;
    const char *errorString;
} client_error_t;

/* see client_err.h for the defines */

client_error_t client_errors[] = {
    { 0, "URL does not begin with \"ldap\" or \"http\"" },
    { 1, "URL does not begin with \"ldap://\"" },
    { 2, "URL missing trailing slash after host or port" },
    { 3, "URL contains an invalid scope" },
    { 4, "URL parsing ran out of memory" },
    { 5, "URL parsing unknown error" },
    { 6, "No attributes defined" },
    { 7, "Too many attributes, only 1 allowed" },
    { 8, "LDAP connection failed" },
    { 9, "Unable to anonymously bind to ldap server" },
    { 10, "ldap_search_st() failed" },
    { 11, "0 or more than 1 matches were returned" },
    { 12, "Unable to get attributes" },
    { 13, "Problem with LDAP credentials" },
    { 14, "Unable to bind to ldap server" },
    { 15, "Unable to enable LDAP client authentication" },
    { 16, "Unable to parse http[s] URL" },
    { 17, "Unable to create client socket" },
    { 18, "DNS lookup failed" },
    { 19, "PR_EnumerateHostEnt failed" },
    { 20, "Unable to connect to remote host" },
    { 21, "Unable to write data to remote server" },
    { 22, "Unable to read data from remote server" },
    { 23, "Out of memory while reading data" }
};

/* Given a URL, determine the type and fetch the appropriate contents and 
 * return them.
 *
 * url - currenly supports HTTP[S] and LDAP[S] URLs
 * timeout - timeout in seconds for network connections, reads and writes
 * len - the length of the data returned
 * RevStatus - contains any error codes and messages
 *
 * NOTE: The caller is responsible for freeing any data that is returned.
 *
 * See the ldap-client.cpp and http-client.cpp for specific URL syntax.
 */
PR_IMPLEMENT(void *)fetch_url(const char * url, int timeout, int * len, RevStatus& status) 
{
    int errnum = -1;
    void * data = NULL;
    
    if (!url) {
        status.setError(REV_ERROR_INVALID_URL_TYPE, client_errors[errnum].errorString);
    }

    if (!status.hasFailed()) {
        if (!PL_strncasecmp(url, "ldap", 4))
            data = ldap_client(url, timeout, len, &errnum);
        else if (!PL_strncasecmp(url, "http", 4))
            data = http_client(url, timeout, len, &errnum);
        else if (!PL_strncasecmp(url, "exec", 4))
            data = exec_client(url, timeout, len, &errnum);
        else
            errnum = CL_URL_UNKNOWN;
    }
    
    if (errnum != -1) {
        status.setError(REV_ERROR_INVALID_URL_TYPE, client_errors[errnum].errorString);
    }

    return data;
}

PR_IMPLEMENT(void)free_url(void* urldata)
{
    PR_ASSERT(urldata);
    free(urldata);
}

